/*_############################################################################
  _## 
  _##  SNMP4J-UNIX-TRANSPORT - UnixDomainAddress.java  
  _## 
  _##  Copyright (C) 2021-2022  Frank Fock (SNMP4J.org)
  _##  
  _##  Licensed under the Apache License, Version 2.0 (the "License");
  _##  you may not use this file except in compliance with the License.
  _##  You may obtain a copy of the License at
  _##  
  _##      http://www.apache.org/licenses/LICENSE-2.0
  _##  
  _##  Unless required by applicable law or agreed to in writing, software
  _##  distributed under the License is distributed on an "AS IS" BASIS,
  _##  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  _##  See the License for the specific language governing permissions and
  _##  limitations under the License.
  _##  
  _##########################################################################*/
package org.snmp4j.transport.unix;

import org.snmp4j.asn1.BERInputStream;
import org.snmp4j.smi.*;
import org.snmp4j.transport.TransportStateListener;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Serial;
import java.net.ProtocolFamily;
import java.net.SocketAddress;
import java.net.StandardProtocolFamily;
import java.net.UnixDomainSocketAddress;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

public class UnixDomainAddress extends SMIAddress {

    @Serial
    private static final long serialVersionUID = -6151626307033468271L;

    private java.net.UnixDomainSocketAddress unixDomainAddress;

    private transient List<TransportStateListener> transportStateListeners;

    public UnixDomainAddress() {
    }

    public UnixDomainAddress(Path unixDomainAddress) {
        this.unixDomainAddress = java.net.UnixDomainSocketAddress.of(unixDomainAddress);
    }

    public UnixDomainAddress(java.net.UnixDomainSocketAddress unixDomainAddress) {
        this.unixDomainAddress = unixDomainAddress;
    }


    public java.net.UnixDomainSocketAddress getUnixDomainAddress() {
        return unixDomainAddress;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        UnixDomainAddress that = (UnixDomainAddress) o;
        return getUnixDomainAddress().equals(that.getUnixDomainAddress());
    }

    @Override
    public int compareTo(Variable variable) {
        return unixDomainAddress.getPath().compareTo(((UnixDomainAddress)variable).unixDomainAddress.getPath());
    }

    @Override
    public int hashCode() {
        return unixDomainAddress.hashCode();
    }

    @Override
    public int getBERLength() {
        return new OctetString(unixDomainAddress.getPath().toString()).getBERLength();
    }

    @Override
    public void decodeBER(BERInputStream berInputStream) throws IOException {
        OctetString os = new OctetString();
        os.decodeBER(berInputStream);
        this.unixDomainAddress = java.net.UnixDomainSocketAddress.of(os.toString());
    }

    @Override
    public void encodeBER(OutputStream outputStream) throws IOException {
        OctetString os = new OctetString(unixDomainAddress.toString());
        os.encodeBER(outputStream);
    }

    @Override
    public int getSyntax() {
        return SMIConstants.SYNTAX_OCTET_STRING;
    }

    @Override
    public String toString() {
        return (unixDomainAddress == null) ? GenericAddress.TYPE_UNIX+":[]" : unixDomainAddress.toString();
    }

    @Override
    public int toInt() {
        throw new UnsupportedOperationException();
    }

    @Override
    public long toLong() {
        throw new UnsupportedOperationException();
    }

    @Override
    public Object clone() {
        return new UnixDomainAddress(unixDomainAddress.getPath());
    }

    @Override
    public OID toSubIndex(boolean impliedLength) {
        return new OctetString(unixDomainAddress.toString()).toSubIndex(impliedLength);
    }

    @Override
    public void fromSubIndex(OID oid, boolean b) {
        OctetString os = OctetString.fromIndex(oid, 0, b);
        this.unixDomainAddress = java.net.UnixDomainSocketAddress.of(Path.of(os.toString()));
    }

    @Override
    public boolean isValid() {
        return isValidUnixDomainSocket(unixDomainAddress.getPath());
    }

    private static boolean isValidUnixDomainSocket(Path path) {
        File socketFile = path.getParent().toFile();
        return socketFile.canExecute() && socketFile.canWrite() && socketFile.isDirectory();
    }

    @Override
    public boolean parseAddress(String s) {
        java.net.UnixDomainSocketAddress unixDomainSocketAddress = java.net.UnixDomainSocketAddress.of(s);
        if (isValidUnixDomainSocket(unixDomainSocketAddress.getPath())) {
            this.unixDomainAddress = unixDomainSocketAddress;
            return true;
        }
        return false;
    }

    @Override
    public void setValue(String s) {
        this.unixDomainAddress = java.net.UnixDomainSocketAddress.of(s);
    }

    @Override
    public void setValue(byte[] bytes) {
        OctetString os = new OctetString(bytes);
        this.unixDomainAddress = java.net.UnixDomainSocketAddress.of(os.toString());
    }

    @Override
    public byte[] toByteArray() {
        return unixDomainAddress.toString().getBytes(StandardCharsets.UTF_8);
    }

    public Path getPath() {
        return unixDomainAddress.getPath();
    }

    @Override
    public ProtocolFamily getFamily() {
        return StandardProtocolFamily.UNIX;
    }

    @Override
    public SocketAddress getSocketAddress() {
        return unixDomainAddress;
    }

}
